@@ -2,9 +2,6 @@
    Copyright (C) 2002, 2003, 2004, 2005, 2006 Bastien Nocera
    Copyright (C) 2003, 2004 Colin Walters <walters@rhythmbox.org>
 
-   Copyright (C) 2005 Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
-   for totem_pl_parser_parse_date ()
-
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
    published by the Free Software Foundation; either version 2 of the
@@ -22,18 +19,9 @@
 
    Authors: Bastien Nocera <hadess@hadess.net>
 
-            for totem_pl_parser_parse_date ():
-            James Livingston  <doclivingston@gmail.com>
-            William Jon McCann  <mccann@jhu.edu>
-            Renato Araujo Oliveira Filho - INdT <renato.filho@indt.org.br>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif /* HAVE_CONFIG_H */
-
-#define _XOPEN_SOURCE
-#include <time.h>
 
 #include <string.h>
 #include <glib.h>
@@ -46,6 +34,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <camel/camel-mime-utils.h>
 
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -506,15 +495,19 @@ totem_pl_parser_relative (const char *url, const char *output)
 		if (newurl[strlen (output_base)] == '/') {
 			base = g_strdup (newurl + strlen (output_base) + 1);
 		} else {
-			base = g_strdup (newurl + strlen (output_base));
+			/* Special case when the output and file are at the root */
+			if (strchr (newurl + strlen (output_base), '/') == NULL)
+				base = g_strdup (newurl + strlen (output_base));
 		}
 		gnome_vfs_uri_unref (uri);
 		g_free (newurl);
 
-		/* And finally unescape the string */
-		newurl = gnome_vfs_unescape_string (base, NULL);
-		g_free (base);
-		base = newurl;
+		if (base != NULL) {
+			/* And finally unescape the string */
+			newurl = gnome_vfs_unescape_string (base, NULL);
+			g_free (base);
+			base = newurl;
+		}
 	}
 
 	g_free (url_base);
@@ -893,6 +886,17 @@ totem_pl_parser_remove_filename (const char *url)
 	return no_file;
 }
 
+static gboolean
+totem_pl_parser_might_be_file (const char *url)
+{
+	const char *mimetype;
+
+	mimetype = gnome_vfs_get_mime_type_for_name (url);
+	if (mimetype == NULL || strcmp (mimetype, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0)
+		return FALSE;
+	return TRUE;
+}
+
 char *
 totem_pl_parser_resolve_url (const char *base, const char *url)
 {
@@ -911,7 +915,13 @@ totem_pl_parser_resolve_url (const char *base, const char *url)
 
 	/* gnome_vfs_uri_append_path is trying to be clever and
 	 * merges paths that look like they're the same */
-	if (url[0] != '/') {
+	if (totem_pl_parser_might_be_file (base) != FALSE) {
+		GnomeVFSURI *new;
+		
+		new = gnome_vfs_uri_new (base_no_frag);
+		base_uri = gnome_vfs_uri_get_parent (new);
+		gnome_vfs_uri_unref (new);
+	} else if (url[0] != '/') {
 		char *newbase = g_strdup_printf ("%s/", base_no_frag);
 		base_uri = gnome_vfs_uri_new (newbase);
 		g_free (newbase);
@@ -1323,106 +1333,39 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 	return -1;
 }
 
+/* FIXME remove when http://bugzilla.gnome.org/show_bug.cgi?id=503029
+ * is fixed */
+static gboolean
+totem_pl_parser_is_iso8601_date (const char *date_str)
+{
+	while (g_ascii_isspace (*date_str))
+		date_str++;
+	if (*date_str == '\0')
+		return FALSE;
+	if (!g_ascii_isdigit (*date_str) && *date_str != '-' && *date_str != '+')
+		return FALSE;
+
+	return TRUE;
+}
+
 guint64
 totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 {
-	struct tm tm;
-	char *result;
+	GTimeVal val;
 
-	/* RFC 2822 date format */
-	result = strptime (date_str, "%a, %d %b %Y %T", &tm);
+	g_return_val_if_fail (date_str != NULL, -1);
 
-	/* same as above, but without comma */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%a %d %b %Y %T", &tm);
+	memset (&val, 0, sizeof(val));
+	/* Try to parse as an ISO8601/RFC3339 date */
+	if (totem_pl_parser_is_iso8601_date (date_str) != FALSE
+	    && g_time_val_from_iso8601 (date_str, &val) != FALSE) {
+		D(g_message ("Parsed duration '%s' using the ISO8601 parser", date_str));
+		return val.tv_sec;
 	}
+	D(g_message ("Failed to parse duration '%s' using the ISO8601 parser", date_str));
 
-	/* close-to-RFC 2822, but with extra 0 */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%a, %d %b %Y 0%T", &tm);
-	}
-
-	/* close-to-RFC 2822, but with no seconds */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%a, %d %b %Y %R", &tm);
-	}
-
-	/* format without weekday */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%d %b %Y %T", &tm);
-	}
-
-	/* reversed day and long month */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%a, %B %d %Y %T", &tm);
-	}
-
-	/* ISO date like */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%Y-%m-%d %T", &tm);
-	}
-
-	/* ISO date like without timezone */
-	if (result == NULL) {
-	memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%Y-%m-%d", &tm);
-	}
-
-	/* Broken weekday short names */
-	if (result == NULL) {
-		char *tmp;
-
-		/* strip off the erroneous weekday */
-		tmp = strstr (date_str, ",");
-		if (tmp != NULL) {
-			tmp++;
-			memset (&tm, 0, sizeof (struct tm));
-			result = strptime (tmp, "%d %b %Y %T", &tm);
-		}
-	}
-
-	/* format with timezone offset from GMT */
-	if (result == NULL) {
-		memset (&tm, 0, sizeof (struct tm));
-		result = strptime (date_str, "%a %b %d %T %z %Y", &tm);
-	}
-
-	/* format with timezone name */
-	if (result == NULL) {
-		char *tmp;
-
-		memset (&tm, 0, sizeof (struct tm));
-
-		/* match first part of time string */
-		result = strptime (date_str, "%a %b %d %T ", &tm);
-
-		/* look for anything with a timezone name-like format
-		   i.e. at least one all caps alphabetical character */
-		if (result != NULL) {
-			size_t n;
-
-			n = strspn(result, "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
-			tmp = result+n;
-
-			/* make sure there was at least one character that matched */
-			if ((tmp != NULL) && n > 0)
-				/* remaining part must be the year */
-				result = strptime (tmp, "%Y", &tm);
-			else
-				result = NULL;
-		}
-	}
-
-	if (result == NULL)
-		D(g_message ("Couldn't parse date '%s'\n", date_str));
-
-	return (guint64) ((result == NULL) ? 0 : mktime (&tm));
+	/* Fall back to RFC 2822 date parsing */
+	return camel_header_decode_date (date_str, NULL);
 }
 
 #endif /* !TOTEM_PL_PARSER_MINI */
